body {
    background-color: White;
    color: Black;
    margin: 0;
    padding: 0;
}

p {
    font-family: serif;
}

h1 {
    font-family:  sans-serif;
    font-size:  120%;
    font-weight:  bold;
}

h2 {
    font-family:  sans-serif;
    font-size:  115%;
}

h3 {
    font-family:  sans-serif;
    font-size:  110%;
}

h4 {
    font-family:  sans-serif;
    font-size:  110%;
    font-style: italic;
}

a:link {
    text-decoration: none;
    color: #336699;
}

a:visited {
    text-decoration: none;
    color: #336699;
}

a:active {
    color: #336699;
    text-decoration: none;
}

a:hover {
    text-decoration: none;
    color: Blue;
}

a img {
    border: 0;
}

table {
    border: 0;
}

td {
    border: 0;
    margin: 0;
    vertical-align: top;
}

th {
    border: 0;
    margin: 0;
}

#Masthead {
    background-color: #336699;
    border: 1pt;
    border-color: #336699;
    padding: 0;
    width: 100%;
}

#PortalLogo {
    background-color: #336699;
    vertical-align: middle;
    width: 15%;
    height: 32px;
}

#PortalTitle {
    background-color: #336699;
    color: White;
    text-align: left;
    vertical-align: middle;
    font-family:  sans-serif;
    font-size: 120%;
    font-weight: bold;
    width: 25%;
    height: 32px;
}

#NavBar {
    background-color: #336699;
    color: #FFFFFF;
    text-align: right;
    vertical-align: middle;
    font-family:  sans-serif;
    font-size: 80%;
    width: 60%;
    height: 32px;
}

#NavBar a:link {
    color: #FFFFFF;
    text-decoration: none;
}

#NavBar a:visited {
    color: #FFFFFF;
    text-decoration: none;
}

#NavBar a:active {
    color: #FFFFFF;
    text-decoration: none;
}

#NavBar a:hover {
    color: #5599CC;
    text-decoration: none;
}

#MemberActionsBox {
    color: White;
    background-color: #5599CC;
    border-top: solid #AAAAAA;
    border-top-width: thin;
}

#ContextActionsBox {
    border-color: #AAAAAA;
    background-color: #AAAAAA;
    padding: 0;
}

.ContextActionsRow {
    color: White;
    background-color: #AAAAAA;
}


.ActionTitle {
    color: Black;
    font-family: sans-serif;
    font-weight: bold;
    font-size: 90%;
}

.ActionLink {
    color: Black;
    font-family: sans-serif;
    font-size: 90%;
}


.ActionLink a:link {
    color: White;
    text-decoration:  none;
    font-weight: bold;
}

.ActionLink a:visited {
    color: White;
    text-decoration:  none;
    font-weight: bold;
}

.ActionLink a:active {
    color: White;
    text-decoration:  none;
    font-weight: bold;
}

.ActionLink a:hover {
    color: #336699;
    text-decoration:  none;
    font-weight: bold;
}

.ObjectStatus {
    font-style: italic;
}

#ForematterCell {
    margin-top: 20px;
}

#DesktopStatusBar {
    color: Red;
    background-color: White;
    font-family: Arial, Verdana, Helvetica, sans-serif;
    font-style: italic;
    font-weight: bold;
    margin-top: 8pt;
}

#DesktopTitle {
    margin-top: 8pt;
}

#DesktopDescription {
    font-style: italic;
    margin-left: 20pt;
    margin-right: 8pt;
}

#Desktop {
    margin: 16px 16px 16px 0;
}

#Permalink {
    font-size: 80%;
}

#ContentByline {
    margin-top: 8pt;
    font-size: 90%;
    font-style: italic;
}

#DiscussionAbove {
    margin-left: 16pt;
    margin-right: 8pt;
    margin-top: 8pt;
    margin-bottom: 8pt;
    font-style: italic;
}

table.FormLayout tr {
    vertical-align: top;
}

table.FormLayout tr th.TextField {
    vertical-align: top;
    color: #000000;
}

table.FormLayout tr td.TextField {
    vertical-align: top;
    color: #000000;
}

table.FormLayout th {
    padding: 1pt 2pt 1pt 2pt;
    width: 100px;
    text-align: right;
    vertical-align: top;
    font-size: 80%;
    font-weight: bold;
}

table.FormLayout td {
    padding: 1pt 2pt 1pt 2pt;
}

table.FormLayout dl.FieldHelp dd {
    font-size: 80%;
    color: #000000;
}

table.FormLayout td.ListName {
    padding: 0pt 2pt 0pt 2pt;
    background-color: #336699;
    color: #FFFFFF;
    font-weight: bold;
    vertical-align: middle;
}

table.FormLayout td.ListDefinition {
    font-style: italic;
    color: #000000;
}

table.ContentsList tr td {
    font-size: 100%;
}

table.ContentsList tr td img {
    border: 0;
}

table.SearchResults {
    width: auto;
}

table.SearchResults tr th {
    text-align: left;
}

table.SearchResults tr td img {
    border: 0;
}

table.Wizard {
    width: auto;
}

table.Wizard tr {
    vertical-align: top;
}

table.Wizard tr th {
    text-align: right;
}

div.NewsBar {
    text-align: right;
}

table.NewsItems {
    border: 0;
}

td.NewsBorder {
    background-color: #336699;
}

td.NewsListing {
    background-color: #336699;
    color:  #FFFFFF;
    font-size: 80%;
    font-weight: bold;
}

a.NewsListing{
    background-color: #336699;
    color:  #FFFFFF;
    font-weight: bold;
}

td.NewsByLine {
    background-color:  #CCCCCC;
    color:  #336699;
    font-size: 80%;
    text-align: left;
    vertical-align: top;
}

.NewsLeadin {
    background-color: #FFFFFF;
    color:  #336699;
}

.NewsByLine {
    background-color:  #336699;
    color:  #FFFFFF;
}

.NewsDateline {
    background-color:  #336699;
    color:  #FFFFFF;
}

td.NewsTitle {
    background-color: #336699;
    color: #FFFFFF;
    text-align: center;
    font-size: 90%;
    font-weight: bold;
}

.NewsItemRow {
    background-color: #AAAAAA;
    font-size: 70%;
}

p.NewsHeadline {
    background-color:  #336699;
}

.BatchListing {
    text-indent: -25px;
    margin: 7pt 0pt 1pt 25px;
}

.BatchListing img {
    vertical-align: baseline;
}

.BatchListing span {
    font: bold 8pt Verdana, sans-serif;
    color: #aaaaaa;
}

.BatchTable {
    border-spacing: 0;
    border-collapse: collapse;
}

.BatchTable th {
    padding: 1pt 2pt 1pt 2pt;

}

.BatchTable td {
    padding: 1pt 2pt 1pt 2pt;
}

.BatchTable tr.list-header {
    background-color: #aaaaaa;
    text-align: left;
    font: 90%/120% sans-serif;
}

.BatchTable tr.list-header a:link {
    color: #ffffff;
    text-decoration: none;
}

.BatchTable tr.list-header a:visited {
    color: #ffffff;
    text-decoration: none;
}

.BatchTable tr.list-header a:active {
    color: #ffffff;
    text-decoration: none;
}

.BatchTable tr.list-header a:hover {
    color: #336699;
    text-decoration: none;
}

.BatchTable tr.row-normal {
    background-color: #ffffff;
}

.BatchTable tr.row-hilite {
    background-color: #efefef;
}

.FormButtons {
    margin-top: 8pt;
}

.form            { width: 720px; background-color: #efef8f; }
.widgets         { width: 720px; float: left; }
.widget          { width: 720px; float: left; margin-top: 6px;
                   min-height: 30px; height: auto !important; height: 30px;
                   font-family: sans-serif; font-size: 90%;
                   background-color: #efefef; }
.widget.split    { width: 360px; }
.widget label    { width: 108px; float: left; padding: 0 4px;
                   min-height: 17px; height: auto !important; height: 17px;
                   text-align: right; font-size: 80%; font-weight: bold; }
.widget .data label { width: auto; float: none;
                      font-size: inherit; font-weight: normal; }
.widget .field   { width: 604px; float: left;
                   font-family: inherit; font-size: inherit; }
.widget.split .field { width: 244px; }
.widget .data    { width: 600px; padding: 0 2px;
                   min-height: 30px; height: auto !important; height: 30px;
                   background-color: #ffffff; }
.widget.split .data { width: 240px; }
.widget input.fileType,
.widget input.textType,
.widget select,
.widget textarea { width: 596px; margin: 0 -2px; padding: 0 2px;
                   font-family: inherit; font-size: inherit; }
.widget select   { padding: 0 }
.widget input.textType {
                   min-height: 30px; height: auto !important; height: 24px; }
.widget.split input.textType,
.widget.split textarea { width: 236px; }
.widget.split select { width: 244px; }
.widget ul       { margin: 0 0 0 10px; padding: 0 0 0 10px;
                   list-style-type: disc; }
.widget li,
.widget pre      { margin: 0; }
.widget .error   { display: block;
                   font-size: 80%; font-weight: bold; color: red; }
.errors          { font-size: 90%; font-weight: bold; color: red; }
.errors li       { font-family: sans-serif;
                   font-size: 80%; font-weight: bold; color: red; }
.clear           { clear: both; padding-top: 6px; }
.buttons         { min-width: 604px; margin: 0 0 0 116px; }
.button          { margin-bottom: 6px; }
